import 'dart:async';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

class IDKitMultiFingerRecognizer extends StatefulWidget {
  const IDKitMultiFingerRecognizer({
    super.key,
    this.child,
    this.behavior,
    this.excludeFromSemantics = false,
    this.semantics,
    this.fingerRecognizerCall,
  });

  /// Detect sub-components of the finger component
  final Widget? child;

  /// Detect sub-widget touch form of finger widget
  final HitTestBehavior? behavior;
  final bool excludeFromSemantics;
  final SemanticsGestureDelegate? semantics;

  /// Callback for detecting finger events
  final Function(int)? fingerRecognizerCall;

  @override
  State<IDKitMultiFingerRecognizer> createState() =>
      _IDKitMultiFingerRecognizer();
}

class _IDKitMultiFingerRecognizer extends State<IDKitMultiFingerRecognizer> {
  /// finger count
  late int count = 0;
  late int subCount = 0;
  late bool isSub = false;

  /// Finger Recognition Monitor Object
  late final StreamController<int> streamController = StreamController<int>();
  late final StreamController<int> subStreamController =
      StreamController<int>();

  @override
  void initState() {
    if (widget.fingerRecognizerCall != null) {
      streamController.stream
          .listen((event) => widget.fingerRecognizerCall!.call(event));
      subStreamController.stream.listen((event) {
        if (isSub) {
          if (subCount == count) {
            count = 0;
            subCount = 0;
            isSub = false;
          }
        }
      });
    }

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return RawGestureDetector(
      key: widget.key,
      gestures: {
        SerialTapGestureRecognizer:
            GestureRecognizerFactoryWithHandlers<SerialTapGestureRecognizer>(
          () => SerialTapGestureRecognizer(),
          (SerialTapGestureRecognizer instance) {
            /// all fingers down
            instance.onSerialTapDown =
                (SerialTapDownDetails details) => count++;

            /// finger press and slide
            instance.onSerialTapCancel = (details) {
              subCount++;
              if (!isSub) {
                subStreamController.add(subCount);
                isSub = true;
              } else {
                subStreamController.add(subCount);
              }
            };

            /// all fingers up
            instance.onSerialTapUp = (SerialTapUpDetails details) {
              if (details.count == 1 && widget.fingerRecognizerCall != null) {
                streamController.add(count);
              }
              count = 0;
              isSub = false;
              subCount = 0;
            };
          },
        ),
      },
      excludeFromSemantics: widget.excludeFromSemantics,
      semantics: widget.semantics,
      behavior: widget.behavior,
      child: widget.child,
    );
  }

  @override
  void dispose() {
    if (widget.fingerRecognizerCall != null) {
      streamController.close();
      subStreamController.close();
    }
    super.dispose();
  }
}
